/*
 *  Copyright (C) 2007  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <atompub/atom.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

void
cgi_serve_request (AtomCtx *ctx)
{
  char *method = getenv ("REQUEST_METHOD");
  char *path = getenv ("PATH_INFO");
  if (method == NULL)
    return;
  if (path == NULL)
    {
      if ((path = atom_config_get_str (ctx, "cgi", "index")) == NULL)
        {
          /* We do not want to disclose our configuration is empty.
           * The requester cannot distinguish an empty configuration
           * from a non-existent index.
           */
          fprintf (stdout, "Status: 404 Not Found\n\n");
          return;
        }
    }
  if (!strcmp (method, "GET"))
    {
      AtomID *id;
      AtomResource *atom;
      AtomError *error;
      id = atom_id_new (path);
      atom = atom_retrieve_resource (ctx, id);
      atom_id_delete (id);
      if (atom)
	{
	  char * str;
	  size_t len;
	  char *header = "Content-type: application/atom+xml\n\n";
	  write (1, header, strlen (header));
	  atom_resource_string (atom, &str, &len);
	  write (1, str, len);
	  g_free (str);
	  atom_resource_delete (atom);
	}
      else if ((error = atom_error_get (ctx)) != NULL)
	{
	  int code = atom_error_code (error);
	  char *message = atom_error_message (error);
	  fprintf (stdout, "Status: %d %s\n\n%s\n", code, message, message);
	}
      else
        {
	  fprintf (stdout, "Status: 500 Server error\n\nServer error\n");
	}
    }
  else
    {
      fprintf (stdout, "Status: 501 Not Implemented\n\n");
    }
}

AtomFrontend *
cgi_frontend (void)
{
  AtomFrontend *frontend;
  frontend = atom_frontend_new ();
  atom_frontend_map_entries_set (frontend, atom_map_frontend_requests);
  return frontend;
}
